@@ -6,19 +6,38 @@
 #ifndef _INC_TYPES_H
 #define _INC_TYPES_H
 
+#if ATTACK_MODE == 9
+#define SALT_POS       (pws_pos + gid)
+#define DIGESTS_CNT    1
+#define DIGESTS_OFFSET (pws_pos + gid)
+#else
+#define SALT_POS       salt_pos_host
+#define DIGESTS_CNT    digests_cnt_host
+#define DIGESTS_OFFSET digests_offset_host
+#endif
+
 #ifdef IS_CUDA
-//https://docs.nvidia.com/cuda/nvrtc/index.html#integer-size
-typedef unsigned char      uchar;
-typedef unsigned short     ushort;
-typedef unsigned int       uint;
-typedef unsigned long long ulong;
+// https://docs.nvidia.com/cuda/nvrtc/index.html#integer-size
+typedef unsigned char       uchar;
+typedef unsigned short      ushort;
+typedef unsigned int        uint;
+typedef unsigned long       ulong;
+typedef unsigned long long  ullong;
+#endif
+
+#ifdef IS_OPENCL
+typedef ulong   ullong;
+typedef ulong2  ullong2;
+typedef ulong4  ullong4;
+typedef ulong8  ullong8;
+typedef ulong16 ullong16;
 #endif
 
 #ifdef KERNEL_STATIC
 typedef uchar  u8;
 typedef ushort u16;
 typedef uint   u32;
-typedef ulong  u64;
+typedef ullong u64;
 #else
 typedef uint8_t  u8;
 typedef uint16_t u16;
@@ -58,7 +77,7 @@ typedef u64  u64x;
 #define make_u64x (u64)
 
 #else
-#ifdef IS_CUDA
+#if defined IS_CUDA || defined IS_HIP
 
 #if VECT_SIZE == 2
 
@@ -161,6 +180,9 @@ inline __device__ u32x operator -  (const u32x a, const u32x b) { return u32x ((
 inline __device__ u32x operator *  (const u32x a, const u32  b) { return u32x ((a.s0 *  b),    (a.s1 *  b)   );  }
 inline __device__ u32x operator *  (const u32x a, const u32x b) { return u32x ((a.s0 *  b.s0), (a.s1 *  b.s1));  }
 
+inline __device__ u32x operator %  (const u32x a, const u32  b) { return u32x ((a.s0 %  b),    (a.s1 %  b)   );  }
+inline __device__ u32x operator %  (const u32x a, const u32x b) { return u32x ((a.s0 %  b.s0), (a.s1 %  b.s1));  }
+
 inline __device__ u32x operator ~  (const u32x a) { return u32x (~a.s0, ~a.s1); }
 
 inline __device__ bool operator != (const u64x a, const u64  b) { return ((a.s0 != b)    && (a.s1 != b));    }
@@ -214,6 +236,9 @@ inline __device__ u64x operator -  (const u64x a, const u64x b) { return u64x ((
 inline __device__ u64x operator *  (const u64x a, const u64  b) { return u64x ((a.s0 *  b),    (a.s1 *  b)   );  }
 inline __device__ u64x operator *  (const u64x a, const u64x b) { return u64x ((a.s0 *  b.s0), (a.s1 *  b.s1));  }
 
+inline __device__ u64x operator %  (const u64x a, const u64  b) { return u64x ((a.s0 %  b),    (a.s1 %  b)   );  }
+inline __device__ u64x operator %  (const u64x a, const u64x b) { return u64x ((a.s0 %  b.s0), (a.s1 %  b.s1));  }
+
 inline __device__ u64x operator ~  (const u64x a) { return u64x (~a.s0, ~a.s1); }
 
 #endif
@@ -327,6 +352,9 @@ inline __device__ u32x operator -  (const u32x a, const u32x b) { return u32x ((
 inline __device__ u32x operator *  (const u32x a, const u32  b) { return u32x ((a.s0 *  b),    (a.s1 *  b)   , (a.s2 *  b),    (a.s3 *  b)   );  }
 inline __device__ u32x operator *  (const u32x a, const u32x b) { return u32x ((a.s0 *  b.s0), (a.s1 *  b.s1), (a.s2 *  b.s2), (a.s3 *  b.s3));  }
 
+inline __device__ u32x operator %  (const u32x a, const u32  b) { return u32x ((a.s0 %  b),    (a.s1 %  b)   , (a.s2 %  b),    (a.s3 %  b)   );  }
+inline __device__ u32x operator %  (const u32x a, const u32x b) { return u32x ((a.s0 %  b.s0), (a.s1 %  b.s1), (a.s2 %  b.s2), (a.s3 %  b.s3));  }
+
 inline __device__ u32x operator ~  (const u32x a) { return u32x (~a.s0, ~a.s1, ~a.s2, ~a.s3); }
 
 inline __device__ bool operator != (const u64x a, const u64  b) { return ((a.s0 != b)    && (a.s1 != b)    && (a.s2 != b)    && (a.s3 != b)   ); }
@@ -380,6 +408,9 @@ inline __device__ u64x operator -  (const u64x a, const u64x b) { return u64x ((
 inline __device__ u64x operator *  (const u64x a, const u64  b) { return u64x ((a.s0 *  b),    (a.s1 *  b)   , (a.s2 *  b),    (a.s3 *  b)   );  }
 inline __device__ u64x operator *  (const u64x a, const u64x b) { return u64x ((a.s0 *  b.s0), (a.s1 *  b.s1), (a.s2 *  b.s2), (a.s3 *  b.s3));  }
 
+inline __device__ u64x operator %  (const u64x a, const u32  b) { return u64x ((a.s0 %  b),    (a.s1 %  b)   , (a.s2 %  b),    (a.s3 %  b)   );  }
+inline __device__ u64x operator %  (const u64x a, const u64x b) { return u64x ((a.s0 %  b.s0), (a.s1 %  b.s1), (a.s2 %  b.s2), (a.s3 %  b.s3));  }
+
 inline __device__ u64x operator ~  (const u64x a) { return u64x (~a.s0, ~a.s1, ~a.s2, ~a.s3); }
 
 #endif
@@ -509,6 +540,9 @@ inline __device__ u32x operator -  (const u32x a, const u32x b) { return u32x ((
 inline __device__ u32x operator *  (const u32x a, const u32  b) { return u32x ((a.s0 *  b),    (a.s1 *  b)   , (a.s2 *  b),    (a.s3 *  b)   , (a.s4 *  b),    (a.s5 *  b)   , (a.s6 *  b),    (a.s7 *  b)   );  }
 inline __device__ u32x operator *  (const u32x a, const u32x b) { return u32x ((a.s0 *  b.s0), (a.s1 *  b.s1), (a.s2 *  b.s2), (a.s3 *  b.s3), (a.s4 *  b.s4), (a.s5 *  b.s5), (a.s6 *  b.s6), (a.s7 *  b.s7));  }
 
+inline __device__ u32x operator %  (const u32x a, const u32  b) { return u32x ((a.s0 %  b),    (a.s1 %  b)   , (a.s2 %  b),    (a.s3 %  b)   , (a.s4 %  b),    (a.s5 %  b)   , (a.s6 %  b),    (a.s7 %  b)   );  }
+inline __device__ u32x operator %  (const u32x a, const u32x b) { return u32x ((a.s0 %  b.s0), (a.s1 %  b.s1), (a.s2 %  b.s2), (a.s3 %  b.s3), (a.s4 %  b.s4), (a.s5 %  b.s5), (a.s6 %  b.s6), (a.s7 %  b.s7));  }
+
 inline __device__ u32x operator ~  (const u32x a) { return u32x (~a.s0, ~a.s1, ~a.s2, ~a.s3, ~a.s4, ~a.s5, ~a.s6, ~a.s7); }
 
 inline __device__ bool operator != (const u64x a, const u64  b) { return ((a.s0 != b)    && (a.s1 != b)    && (a.s2 != b)    && (a.s3 != b)    && (a.s4 != b)    && (a.s5 != b)    && (a.s6 != b)    && (a.s7 != b)   ); }
@@ -562,6 +596,9 @@ inline __device__ u64x operator -  (const u64x a, const u64x b) { return u64x ((
 inline __device__ u64x operator *  (const u64x a, const u64  b) { return u64x ((a.s0 *  b),    (a.s1 *  b)   , (a.s2 *  b),    (a.s3 *  b)   , (a.s4 *  b),    (a.s5 *  b)   , (a.s6 *  b),    (a.s7 *  b)   );  }
 inline __device__ u64x operator *  (const u64x a, const u64x b) { return u64x ((a.s0 *  b.s0), (a.s1 *  b.s1), (a.s2 *  b.s2), (a.s3 *  b.s3), (a.s4 *  b.s4), (a.s5 *  b.s5), (a.s6 *  b.s6), (a.s7 *  b.s7));  }
 
+inline __device__ u64x operator %  (const u64x a, const u64  b) { return u64x ((a.s0 %  b),    (a.s1 %  b)   , (a.s2 %  b),    (a.s3 %  b)   , (a.s4 %  b),    (a.s5 %  b)   , (a.s6 %  b),    (a.s7 %  b)   );  }
+inline __device__ u64x operator %  (const u64x a, const u64x b) { return u64x ((a.s0 %  b.s0), (a.s1 %  b.s1), (a.s2 %  b.s2), (a.s3 %  b.s3), (a.s4 %  b.s4), (a.s5 %  b.s5), (a.s6 %  b.s6), (a.s7 %  b.s7));  }
+
 inline __device__ u64x operator ~  (const u64x a) { return u64x (~a.s0, ~a.s1, ~a.s2, ~a.s3, ~a.s4, ~a.s5, ~a.s6, ~a.s7); }
 
 #endif
@@ -723,6 +760,9 @@ inline __device__ u32x operator -  (const u32x a, const u32x b) { return u32x ((
 inline __device__ u32x operator *  (const u32x a, const u32  b) { return u32x ((a.s0 *  b),    (a.s1 *  b)   , (a.s2 *  b),    (a.s3 *  b)   , (a.s4 *  b),    (a.s5 *  b)   , (a.s6 *  b),    (a.s7 *  b),    (a.s8 *  b),    (a.s9 *  b)   , (a.sa *  b),    (a.sb *  b)   , (a.sc *  b),    (a.sd *  b)   , (a.se *  b),    (a.sf *  b)   );  }
 inline __device__ u32x operator *  (const u32x a, const u32x b) { return u32x ((a.s0 *  b.s0), (a.s1 *  b.s1), (a.s2 *  b.s2), (a.s3 *  b.s3), (a.s4 *  b.s4), (a.s5 *  b.s5), (a.s6 *  b.s6), (a.s7 *  b.s7), (a.s8 *  b.s8), (a.s9 *  b.s9), (a.sa *  b.sa), (a.sb *  b.sb), (a.sc *  b.sc), (a.sd *  b.sd), (a.se *  b.se), (a.sf *  b.sf));  }
 
+inline __device__ u32x operator %  (const u32x a, const u32  b) { return u32x ((a.s0 %  b),    (a.s1 %  b)   , (a.s2 %  b),    (a.s3 %  b)   , (a.s4 %  b),    (a.s5 %  b)   , (a.s6 %  b),    (a.s7 %  b),    (a.s8 %  b),    (a.s9 %  b)   , (a.sa %  b),    (a.sb %  b)   , (a.sc %  b),    (a.sd %  b)   , (a.se %  b),    (a.sf %  b)   );  }
+inline __device__ u32x operator %  (const u32x a, const u32x b) { return u32x ((a.s0 %  b.s0), (a.s1 %  b.s1), (a.s2 %  b.s2), (a.s3 %  b.s3), (a.s4 %  b.s4), (a.s5 %  b.s5), (a.s6 %  b.s6), (a.s7 %  b.s7), (a.s8 %  b.s8), (a.s9 %  b.s9), (a.sa %  b.sa), (a.sb %  b.sb), (a.sc %  b.sc), (a.sd %  b.sd), (a.se %  b.se), (a.sf %  b.sf));  }
+
 inline __device__ u32x operator ~  (const u32x a) { return u32x (~a.s0, ~a.s1, ~a.s2, ~a.s3, ~a.s4, ~a.s5, ~a.s6, ~a.s7, ~a.s8, ~a.s9, ~a.sa, ~a.sb, ~a.sc, ~a.sd, ~a.se, ~a.sf); }
 
 inline __device__ bool operator != (const u64x a, const u64  b) { return ((a.s0 != b)    && (a.s1 != b)    && (a.s2 != b)    && (a.s3 != b)    && (a.s4 != b)    && (a.s5 != b)    && (a.s6 != b)    && (a.s7 != b)    && (a.s8 != b)    && (a.s9 != b)    && (a.sa != b)    && (a.sb != b)    && (a.sc != b)    && (a.sd != b)    && (a.se != b)    && (a.sf != b)   ); }
@@ -776,6 +816,9 @@ inline __device__ u64x operator -  (const u64x a, const u64x b) { return u64x ((
 inline __device__ u64x operator *  (const u64x a, const u64  b) { return u64x ((a.s0 *  b),    (a.s1 *  b)   , (a.s2 *  b),    (a.s3 *  b)   , (a.s4 *  b),    (a.s5 *  b)   , (a.s6 *  b),    (a.s7 *  b),    (a.s8 *  b),    (a.s9 *  b)   , (a.sa *  b),    (a.sb *  b)   , (a.sc *  b),    (a.sd *  b)   , (a.se *  b),    (a.sf *  b)   );  }
 inline __device__ u64x operator *  (const u64x a, const u64x b) { return u64x ((a.s0 *  b.s0), (a.s1 *  b.s1), (a.s2 *  b.s2), (a.s3 *  b.s3), (a.s4 *  b.s4), (a.s5 *  b.s5), (a.s6 *  b.s6), (a.s7 *  b.s7), (a.s8 *  b.s8), (a.s9 *  b.s9), (a.sa *  b.sa), (a.sb *  b.sb), (a.sc *  b.sc), (a.sd *  b.sd), (a.se *  b.se), (a.sf *  b.sf));  }
 
+inline __device__ u64x operator %  (const u64x a, const u64  b) { return u64x ((a.s0 %  b),    (a.s1 %  b)   , (a.s2 %  b),    (a.s3 %  b)   , (a.s4 %  b),    (a.s5 %  b)   , (a.s6 %  b),    (a.s7 %  b),    (a.s8 %  b),    (a.s9 %  b)   , (a.sa %  b),    (a.sb %  b)   , (a.sc %  b),    (a.sd %  b)   , (a.se %  b),    (a.sf %  b)   );  }
+inline __device__ u64x operator %  (const u64x a, const u64x b) { return u64x ((a.s0 %  b.s0), (a.s1 %  b.s1), (a.s2 %  b.s2), (a.s3 %  b.s3), (a.s4 %  b.s4), (a.s5 %  b.s5), (a.s6 %  b.s6), (a.s7 %  b.s7), (a.s8 %  b.s8), (a.s9 %  b.s9), (a.sa %  b.sa), (a.sb %  b.sb), (a.sc %  b.sc), (a.sd %  b.sd), (a.se %  b.se), (a.sf %  b.sf));  }
+
 inline __device__ u64x operator ~  (const u64x a) { return u64x (~a.s0, ~a.s1, ~a.s2, ~a.s3, ~a.s4, ~a.s5, ~a.s6, ~a.s7, ~a.s8, ~a.s9, ~a.sa, ~a.sb, ~a.sc, ~a.sd, ~a.se, ~a.sf); }
 
 #endif
@@ -791,10 +834,10 @@ typedef __device_builtin__ struct u64x u64x;
 #define make_u64x u64x
 
 #else
-typedef VTYPE(uchar,  VECT_SIZE)  u8x;
+typedef VTYPE(uchar,  VECT_SIZE) u8x;
 typedef VTYPE(ushort, VECT_SIZE) u16x;
 typedef VTYPE(uint,   VECT_SIZE) u32x;
-typedef VTYPE(ulong,  VECT_SIZE) u64x;
+typedef VTYPE(ullong, VECT_SIZE) u64x;
 
 #define make_u8x  (u8x)
 #define make_u16x (u16x)
@@ -870,30 +913,30 @@ typedef union vconv64
 
 typedef enum siphash_constants
 {
-  SIPHASHM_0=0x736f6d6570736575,
-  SIPHASHM_1=0x646f72616e646f6d,
-  SIPHASHM_2=0x6c7967656e657261,
-  SIPHASHM_3=0x7465646279746573
+  SIPHASHM_0=0x736f6d6570736575UL,
+  SIPHASHM_1=0x646f72616e646f6dUL,
+  SIPHASHM_2=0x6c7967656e657261UL,
+  SIPHASHM_3=0x7465646279746573UL
 
 } siphash_constants_t;
 
 typedef enum bcrypt_constants
 {
-  BCRYPTM_0=0x4F727068,
-  BCRYPTM_1=0x65616E42,
-  BCRYPTM_2=0x65686F6C,
-  BCRYPTM_3=0x64657253,
-  BCRYPTM_4=0x63727944,
-  BCRYPTM_5=0x6F756274u
+  BCRYPTM_0=0x4f727068U,
+  BCRYPTM_1=0x65616e42U,
+  BCRYPTM_2=0x65686f6cU,
+  BCRYPTM_3=0x64657253U,
+  BCRYPTM_4=0x63727944U,
+  BCRYPTM_5=0x6f756274U
 
 } bcrypt_constants_t;
 
 typedef enum md4_constants
 {
-  MD4M_A=0x67452301,
-  MD4M_B=0xefcdab89,
-  MD4M_C=0x98badcfe,
-  MD4M_D=0x10325476,
+  MD4M_A=0x67452301U,
+  MD4M_B=0xefcdab89U,
+  MD4M_C=0x98badcfeU,
+  MD4M_D=0x10325476U,
 
   MD4S00=3,
   MD4S01=7,
@@ -908,18 +951,18 @@ typedef enum md4_constants
   MD4S22=11,
   MD4S23=15,
 
-  MD4C00=0x00000000,
-  MD4C01=0x5a827999,
-  MD4C02=0x6ed9eba1u
+  MD4C00=0x00000000U,
+  MD4C01=0x5a827999U,
+  MD4C02=0x6ed9eba1U
 
 } md4_constants_t;
 
 typedef enum md5_constants
 {
-  MD5M_A=0x67452301,
-  MD5M_B=0xefcdab89,
-  MD5M_C=0x98badcfe,
-  MD5M_D=0x10325476,
+  MD5M_A=0x67452301U,
+  MD5M_B=0xefcdab89U,
+  MD5M_C=0x98badcfeU,
+  MD5M_D=0x10325476U,
 
   MD5S00=7,
   MD5S01=12,
@@ -938,368 +981,368 @@ typedef enum md5_constants
   MD5S32=15,
   MD5S33=21,
 
-  MD5C00=0xd76aa478,
-  MD5C01=0xe8c7b756,
-  MD5C02=0x242070db,
-  MD5C03=0xc1bdceee,
-  MD5C04=0xf57c0faf,
-  MD5C05=0x4787c62a,
-  MD5C06=0xa8304613,
-  MD5C07=0xfd469501,
-  MD5C08=0x698098d8,
-  MD5C09=0x8b44f7af,
-  MD5C0a=0xffff5bb1,
-  MD5C0b=0x895cd7be,
-  MD5C0c=0x6b901122,
-  MD5C0d=0xfd987193,
-  MD5C0e=0xa679438e,
-  MD5C0f=0x49b40821,
-  MD5C10=0xf61e2562,
-  MD5C11=0xc040b340,
-  MD5C12=0x265e5a51,
-  MD5C13=0xe9b6c7aa,
-  MD5C14=0xd62f105d,
-  MD5C15=0x02441453,
-  MD5C16=0xd8a1e681,
-  MD5C17=0xe7d3fbc8,
-  MD5C18=0x21e1cde6,
-  MD5C19=0xc33707d6,
-  MD5C1a=0xf4d50d87,
-  MD5C1b=0x455a14ed,
-  MD5C1c=0xa9e3e905,
-  MD5C1d=0xfcefa3f8,
-  MD5C1e=0x676f02d9,
-  MD5C1f=0x8d2a4c8a,
-  MD5C20=0xfffa3942,
-  MD5C21=0x8771f681,
-  MD5C22=0x6d9d6122,
-  MD5C23=0xfde5380c,
-  MD5C24=0xa4beea44,
-  MD5C25=0x4bdecfa9,
-  MD5C26=0xf6bb4b60,
-  MD5C27=0xbebfbc70,
-  MD5C28=0x289b7ec6,
-  MD5C29=0xeaa127fa,
-  MD5C2a=0xd4ef3085,
-  MD5C2b=0x04881d05,
-  MD5C2c=0xd9d4d039,
-  MD5C2d=0xe6db99e5,
-  MD5C2e=0x1fa27cf8,
-  MD5C2f=0xc4ac5665,
-  MD5C30=0xf4292244,
-  MD5C31=0x432aff97,
-  MD5C32=0xab9423a7,
-  MD5C33=0xfc93a039,
-  MD5C34=0x655b59c3,
-  MD5C35=0x8f0ccc92,
-  MD5C36=0xffeff47d,
-  MD5C37=0x85845dd1,
-  MD5C38=0x6fa87e4f,
-  MD5C39=0xfe2ce6e0,
-  MD5C3a=0xa3014314,
-  MD5C3b=0x4e0811a1,
-  MD5C3c=0xf7537e82,
-  MD5C3d=0xbd3af235,
-  MD5C3e=0x2ad7d2bb,
-  MD5C3f=0xeb86d391u
+  MD5C00=0xd76aa478U,
+  MD5C01=0xe8c7b756U,
+  MD5C02=0x242070dbU,
+  MD5C03=0xc1bdceeeU,
+  MD5C04=0xf57c0fafU,
+  MD5C05=0x4787c62aU,
+  MD5C06=0xa8304613U,
+  MD5C07=0xfd469501U,
+  MD5C08=0x698098d8U,
+  MD5C09=0x8b44f7afU,
+  MD5C0a=0xffff5bb1U,
+  MD5C0b=0x895cd7beU,
+  MD5C0c=0x6b901122U,
+  MD5C0d=0xfd987193U,
+  MD5C0e=0xa679438eU,
+  MD5C0f=0x49b40821U,
+  MD5C10=0xf61e2562U,
+  MD5C11=0xc040b340U,
+  MD5C12=0x265e5a51U,
+  MD5C13=0xe9b6c7aaU,
+  MD5C14=0xd62f105dU,
+  MD5C15=0x02441453U,
+  MD5C16=0xd8a1e681U,
+  MD5C17=0xe7d3fbc8U,
+  MD5C18=0x21e1cde6U,
+  MD5C19=0xc33707d6U,
+  MD5C1a=0xf4d50d87U,
+  MD5C1b=0x455a14edU,
+  MD5C1c=0xa9e3e905U,
+  MD5C1d=0xfcefa3f8U,
+  MD5C1e=0x676f02d9U,
+  MD5C1f=0x8d2a4c8aU,
+  MD5C20=0xfffa3942U,
+  MD5C21=0x8771f681U,
+  MD5C22=0x6d9d6122U,
+  MD5C23=0xfde5380cU,
+  MD5C24=0xa4beea44U,
+  MD5C25=0x4bdecfa9U,
+  MD5C26=0xf6bb4b60U,
+  MD5C27=0xbebfbc70U,
+  MD5C28=0x289b7ec6U,
+  MD5C29=0xeaa127faU,
+  MD5C2a=0xd4ef3085U,
+  MD5C2b=0x04881d05U,
+  MD5C2c=0xd9d4d039U,
+  MD5C2d=0xe6db99e5U,
+  MD5C2e=0x1fa27cf8U,
+  MD5C2f=0xc4ac5665U,
+  MD5C30=0xf4292244U,
+  MD5C31=0x432aff97U,
+  MD5C32=0xab9423a7U,
+  MD5C33=0xfc93a039U,
+  MD5C34=0x655b59c3U,
+  MD5C35=0x8f0ccc92U,
+  MD5C36=0xffeff47dU,
+  MD5C37=0x85845dd1U,
+  MD5C38=0x6fa87e4fU,
+  MD5C39=0xfe2ce6e0U,
+  MD5C3a=0xa3014314U,
+  MD5C3b=0x4e0811a1U,
+  MD5C3c=0xf7537e82U,
+  MD5C3d=0xbd3af235U,
+  MD5C3e=0x2ad7d2bbU,
+  MD5C3f=0xeb86d391U
 
 } md5_constants_t;
 
 typedef enum sha1_constants
 {
-  SHA1M_A=0x67452301,
-  SHA1M_B=0xefcdab89,
-  SHA1M_C=0x98badcfe,
-  SHA1M_D=0x10325476,
-  SHA1M_E=0xc3d2e1f0,
+  SHA1M_A=0x67452301U,
+  SHA1M_B=0xefcdab89U,
+  SHA1M_C=0x98badcfeU,
+  SHA1M_D=0x10325476U,
+  SHA1M_E=0xc3d2e1f0U,
 
-  SHA1C00=0x5a827999,
-  SHA1C01=0x6ed9eba1,
-  SHA1C02=0x8f1bbcdc,
-  SHA1C03=0xca62c1d6u
+  SHA1C00=0x5a827999U,
+  SHA1C01=0x6ed9eba1U,
+  SHA1C02=0x8f1bbcdcU,
+  SHA1C03=0xca62c1d6U
 
 } sha1_constants_t;
 
 typedef enum sha2_32_constants
 {
   // SHA-224 Initial Hash Values
-  SHA224M_A=0xc1059ed8,
-  SHA224M_B=0x367cd507,
-  SHA224M_C=0x3070dd17,
-  SHA224M_D=0xf70e5939,
-  SHA224M_E=0xffc00b31,
-  SHA224M_F=0x68581511,
-  SHA224M_G=0x64f98fa7,
-  SHA224M_H=0xbefa4fa4,
+  SHA224M_A=0xc1059ed8U,
+  SHA224M_B=0x367cd507U,
+  SHA224M_C=0x3070dd17U,
+  SHA224M_D=0xf70e5939U,
+  SHA224M_E=0xffc00b31U,
+  SHA224M_F=0x68581511U,
+  SHA224M_G=0x64f98fa7U,
+  SHA224M_H=0xbefa4fa4U,
 
   // SHA-224 Constants
-  SHA224C00=0x428a2f98,
-  SHA224C01=0x71374491,
-  SHA224C02=0xb5c0fbcf,
-  SHA224C03=0xe9b5dba5,
-  SHA224C04=0x3956c25b,
-  SHA224C05=0x59f111f1,
-  SHA224C06=0x923f82a4,
-  SHA224C07=0xab1c5ed5,
-  SHA224C08=0xd807aa98,
-  SHA224C09=0x12835b01,
-  SHA224C0a=0x243185be,
-  SHA224C0b=0x550c7dc3,
-  SHA224C0c=0x72be5d74,
-  SHA224C0d=0x80deb1fe,
-  SHA224C0e=0x9bdc06a7,
-  SHA224C0f=0xc19bf174,
-  SHA224C10=0xe49b69c1,
-  SHA224C11=0xefbe4786,
-  SHA224C12=0x0fc19dc6,
-  SHA224C13=0x240ca1cc,
-  SHA224C14=0x2de92c6f,
-  SHA224C15=0x4a7484aa,
-  SHA224C16=0x5cb0a9dc,
-  SHA224C17=0x76f988da,
-  SHA224C18=0x983e5152,
-  SHA224C19=0xa831c66d,
-  SHA224C1a=0xb00327c8,
-  SHA224C1b=0xbf597fc7,
-  SHA224C1c=0xc6e00bf3,
-  SHA224C1d=0xd5a79147,
-  SHA224C1e=0x06ca6351,
-  SHA224C1f=0x14292967,
-  SHA224C20=0x27b70a85,
-  SHA224C21=0x2e1b2138,
-  SHA224C22=0x4d2c6dfc,
-  SHA224C23=0x53380d13,
-  SHA224C24=0x650a7354,
-  SHA224C25=0x766a0abb,
-  SHA224C26=0x81c2c92e,
-  SHA224C27=0x92722c85,
-  SHA224C28=0xa2bfe8a1,
-  SHA224C29=0xa81a664b,
-  SHA224C2a=0xc24b8b70,
-  SHA224C2b=0xc76c51a3,
-  SHA224C2c=0xd192e819,
-  SHA224C2d=0xd6990624,
-  SHA224C2e=0xf40e3585,
-  SHA224C2f=0x106aa070,
-  SHA224C30=0x19a4c116,
-  SHA224C31=0x1e376c08,
-  SHA224C32=0x2748774c,
-  SHA224C33=0x34b0bcb5,
-  SHA224C34=0x391c0cb3,
-  SHA224C35=0x4ed8aa4a,
-  SHA224C36=0x5b9cca4f,
-  SHA224C37=0x682e6ff3,
-  SHA224C38=0x748f82ee,
-  SHA224C39=0x78a5636f,
-  SHA224C3a=0x84c87814,
-  SHA224C3b=0x8cc70208,
-  SHA224C3c=0x90befffa,
-  SHA224C3d=0xa4506ceb,
-  SHA224C3e=0xbef9a3f7,
-  SHA224C3f=0xc67178f2,
+  SHA224C00=0x428a2f98U,
+  SHA224C01=0x71374491U,
+  SHA224C02=0xb5c0fbcfU,
+  SHA224C03=0xe9b5dba5U,
+  SHA224C04=0x3956c25bU,
+  SHA224C05=0x59f111f1U,
+  SHA224C06=0x923f82a4U,
+  SHA224C07=0xab1c5ed5U,
+  SHA224C08=0xd807aa98U,
+  SHA224C09=0x12835b01U,
+  SHA224C0a=0x243185beU,
+  SHA224C0b=0x550c7dc3U,
+  SHA224C0c=0x72be5d74U,
+  SHA224C0d=0x80deb1feU,
+  SHA224C0e=0x9bdc06a7U,
+  SHA224C0f=0xc19bf174U,
+  SHA224C10=0xe49b69c1U,
+  SHA224C11=0xefbe4786U,
+  SHA224C12=0x0fc19dc6U,
+  SHA224C13=0x240ca1ccU,
+  SHA224C14=0x2de92c6fU,
+  SHA224C15=0x4a7484aaU,
+  SHA224C16=0x5cb0a9dcU,
+  SHA224C17=0x76f988daU,
+  SHA224C18=0x983e5152U,
+  SHA224C19=0xa831c66dU,
+  SHA224C1a=0xb00327c8U,
+  SHA224C1b=0xbf597fc7U,
+  SHA224C1c=0xc6e00bf3U,
+  SHA224C1d=0xd5a79147U,
+  SHA224C1e=0x06ca6351U,
+  SHA224C1f=0x14292967U,
+  SHA224C20=0x27b70a85U,
+  SHA224C21=0x2e1b2138U,
+  SHA224C22=0x4d2c6dfcU,
+  SHA224C23=0x53380d13U,
+  SHA224C24=0x650a7354U,
+  SHA224C25=0x766a0abbU,
+  SHA224C26=0x81c2c92eU,
+  SHA224C27=0x92722c85U,
+  SHA224C28=0xa2bfe8a1U,
+  SHA224C29=0xa81a664bU,
+  SHA224C2a=0xc24b8b70U,
+  SHA224C2b=0xc76c51a3U,
+  SHA224C2c=0xd192e819U,
+  SHA224C2d=0xd6990624U,
+  SHA224C2e=0xf40e3585U,
+  SHA224C2f=0x106aa070U,
+  SHA224C30=0x19a4c116U,
+  SHA224C31=0x1e376c08U,
+  SHA224C32=0x2748774cU,
+  SHA224C33=0x34b0bcb5U,
+  SHA224C34=0x391c0cb3U,
+  SHA224C35=0x4ed8aa4aU,
+  SHA224C36=0x5b9cca4fU,
+  SHA224C37=0x682e6ff3U,
+  SHA224C38=0x748f82eeU,
+  SHA224C39=0x78a5636fU,
+  SHA224C3a=0x84c87814U,
+  SHA224C3b=0x8cc70208U,
+  SHA224C3c=0x90befffaU,
+  SHA224C3d=0xa4506cebU,
+  SHA224C3e=0xbef9a3f7U,
+  SHA224C3f=0xc67178f2U,
 
   // SHA-256 Initial Hash Values
-  SHA256M_A=0x6a09e667,
-  SHA256M_B=0xbb67ae85,
-  SHA256M_C=0x3c6ef372,
-  SHA256M_D=0xa54ff53a,
-  SHA256M_E=0x510e527f,
-  SHA256M_F=0x9b05688c,
-  SHA256M_G=0x1f83d9ab,
-  SHA256M_H=0x5be0cd19,
+  SHA256M_A=0x6a09e667U,
+  SHA256M_B=0xbb67ae85U,
+  SHA256M_C=0x3c6ef372U,
+  SHA256M_D=0xa54ff53aU,
+  SHA256M_E=0x510e527fU,
+  SHA256M_F=0x9b05688cU,
+  SHA256M_G=0x1f83d9abU,
+  SHA256M_H=0x5be0cd19U,
 
   // SHA-256 Constants
-  SHA256C00=0x428a2f98,
-  SHA256C01=0x71374491,
-  SHA256C02=0xb5c0fbcf,
-  SHA256C03=0xe9b5dba5,
-  SHA256C04=0x3956c25b,
-  SHA256C05=0x59f111f1,
-  SHA256C06=0x923f82a4,
-  SHA256C07=0xab1c5ed5,
-  SHA256C08=0xd807aa98,
-  SHA256C09=0x12835b01,
-  SHA256C0a=0x243185be,
-  SHA256C0b=0x550c7dc3,
-  SHA256C0c=0x72be5d74,
-  SHA256C0d=0x80deb1fe,
-  SHA256C0e=0x9bdc06a7,
-  SHA256C0f=0xc19bf174,
-  SHA256C10=0xe49b69c1,
-  SHA256C11=0xefbe4786,
-  SHA256C12=0x0fc19dc6,
-  SHA256C13=0x240ca1cc,
-  SHA256C14=0x2de92c6f,
-  SHA256C15=0x4a7484aa,
-  SHA256C16=0x5cb0a9dc,
-  SHA256C17=0x76f988da,
-  SHA256C18=0x983e5152,
-  SHA256C19=0xa831c66d,
-  SHA256C1a=0xb00327c8,
-  SHA256C1b=0xbf597fc7,
-  SHA256C1c=0xc6e00bf3,
-  SHA256C1d=0xd5a79147,
-  SHA256C1e=0x06ca6351,
-  SHA256C1f=0x14292967,
-  SHA256C20=0x27b70a85,
-  SHA256C21=0x2e1b2138,
-  SHA256C22=0x4d2c6dfc,
-  SHA256C23=0x53380d13,
-  SHA256C24=0x650a7354,
-  SHA256C25=0x766a0abb,
-  SHA256C26=0x81c2c92e,
-  SHA256C27=0x92722c85,
-  SHA256C28=0xa2bfe8a1,
-  SHA256C29=0xa81a664b,
-  SHA256C2a=0xc24b8b70,
-  SHA256C2b=0xc76c51a3,
-  SHA256C2c=0xd192e819,
-  SHA256C2d=0xd6990624,
-  SHA256C2e=0xf40e3585,
-  SHA256C2f=0x106aa070,
-  SHA256C30=0x19a4c116,
-  SHA256C31=0x1e376c08,
-  SHA256C32=0x2748774c,
-  SHA256C33=0x34b0bcb5,
-  SHA256C34=0x391c0cb3,
-  SHA256C35=0x4ed8aa4a,
-  SHA256C36=0x5b9cca4f,
-  SHA256C37=0x682e6ff3,
-  SHA256C38=0x748f82ee,
-  SHA256C39=0x78a5636f,
-  SHA256C3a=0x84c87814,
-  SHA256C3b=0x8cc70208,
-  SHA256C3c=0x90befffa,
-  SHA256C3d=0xa4506ceb,
-  SHA256C3e=0xbef9a3f7,
-  SHA256C3f=0xc67178f2,
+  SHA256C00=0x428a2f98U,
+  SHA256C01=0x71374491U,
+  SHA256C02=0xb5c0fbcfU,
+  SHA256C03=0xe9b5dba5U,
+  SHA256C04=0x3956c25bU,
+  SHA256C05=0x59f111f1U,
+  SHA256C06=0x923f82a4U,
+  SHA256C07=0xab1c5ed5U,
+  SHA256C08=0xd807aa98U,
+  SHA256C09=0x12835b01U,
+  SHA256C0a=0x243185beU,
+  SHA256C0b=0x550c7dc3U,
+  SHA256C0c=0x72be5d74U,
+  SHA256C0d=0x80deb1feU,
+  SHA256C0e=0x9bdc06a7U,
+  SHA256C0f=0xc19bf174U,
+  SHA256C10=0xe49b69c1U,
+  SHA256C11=0xefbe4786U,
+  SHA256C12=0x0fc19dc6U,
+  SHA256C13=0x240ca1ccU,
+  SHA256C14=0x2de92c6fU,
+  SHA256C15=0x4a7484aaU,
+  SHA256C16=0x5cb0a9dcU,
+  SHA256C17=0x76f988daU,
+  SHA256C18=0x983e5152U,
+  SHA256C19=0xa831c66dU,
+  SHA256C1a=0xb00327c8U,
+  SHA256C1b=0xbf597fc7U,
+  SHA256C1c=0xc6e00bf3U,
+  SHA256C1d=0xd5a79147U,
+  SHA256C1e=0x06ca6351U,
+  SHA256C1f=0x14292967U,
+  SHA256C20=0x27b70a85U,
+  SHA256C21=0x2e1b2138U,
+  SHA256C22=0x4d2c6dfcU,
+  SHA256C23=0x53380d13U,
+  SHA256C24=0x650a7354U,
+  SHA256C25=0x766a0abbU,
+  SHA256C26=0x81c2c92eU,
+  SHA256C27=0x92722c85U,
+  SHA256C28=0xa2bfe8a1U,
+  SHA256C29=0xa81a664bU,
+  SHA256C2a=0xc24b8b70U,
+  SHA256C2b=0xc76c51a3U,
+  SHA256C2c=0xd192e819U,
+  SHA256C2d=0xd6990624U,
+  SHA256C2e=0xf40e3585U,
+  SHA256C2f=0x106aa070U,
+  SHA256C30=0x19a4c116U,
+  SHA256C31=0x1e376c08U,
+  SHA256C32=0x2748774cU,
+  SHA256C33=0x34b0bcb5U,
+  SHA256C34=0x391c0cb3U,
+  SHA256C35=0x4ed8aa4aU,
+  SHA256C36=0x5b9cca4fU,
+  SHA256C37=0x682e6ff3U,
+  SHA256C38=0x748f82eeU,
+  SHA256C39=0x78a5636fU,
+  SHA256C3a=0x84c87814U,
+  SHA256C3b=0x8cc70208U,
+  SHA256C3c=0x90befffaU,
+  SHA256C3d=0xa4506cebU,
+  SHA256C3e=0xbef9a3f7U,
+  SHA256C3f=0xc67178f2U,
 
 } sha2_32_constants_t;
 
 typedef enum sha2_64_constants
 {
   // SHA-384 Initial Hash Values
-  SHA384M_A=0xcbbb9d5dc1059ed8,
-  SHA384M_B=0x629a292a367cd507,
-  SHA384M_C=0x9159015a3070dd17,
-  SHA384M_D=0x152fecd8f70e5939,
-  SHA384M_E=0x67332667ffc00b31,
-  SHA384M_F=0x8eb44a8768581511,
-  SHA384M_G=0xdb0c2e0d64f98fa7,
-  SHA384M_H=0x47b5481dbefa4fa4,
+  SHA384M_A=0xcbbb9d5dc1059ed8UL,
+  SHA384M_B=0x629a292a367cd507UL,
+  SHA384M_C=0x9159015a3070dd17UL,
+  SHA384M_D=0x152fecd8f70e5939UL,
+  SHA384M_E=0x67332667ffc00b31UL,
+  SHA384M_F=0x8eb44a8768581511UL,
+  SHA384M_G=0xdb0c2e0d64f98fa7UL,
+  SHA384M_H=0x47b5481dbefa4fa4UL,
 
   // SHA-512 Initial Hash Values
-  SHA512M_A=0x6a09e667f3bcc908,
-  SHA512M_B=0xbb67ae8584caa73b,
-  SHA512M_C=0x3c6ef372fe94f82b,
-  SHA512M_D=0xa54ff53a5f1d36f1,
-  SHA512M_E=0x510e527fade682d1,
-  SHA512M_F=0x9b05688c2b3e6c1f,
-  SHA512M_G=0x1f83d9abfb41bd6b,
-  SHA512M_H=0x5be0cd19137e2179,
+  SHA512M_A=0x6a09e667f3bcc908UL,
+  SHA512M_B=0xbb67ae8584caa73bUL,
+  SHA512M_C=0x3c6ef372fe94f82bUL,
+  SHA512M_D=0xa54ff53a5f1d36f1UL,
+  SHA512M_E=0x510e527fade682d1UL,
+  SHA512M_F=0x9b05688c2b3e6c1fUL,
+  SHA512M_G=0x1f83d9abfb41bd6bUL,
+  SHA512M_H=0x5be0cd19137e2179UL,
 
   // SHA-384/512 Constants
-  SHA512C00=0x428a2f98d728ae22,
-  SHA512C01=0x7137449123ef65cd,
-  SHA512C02=0xb5c0fbcfec4d3b2f,
-  SHA512C03=0xe9b5dba58189dbbc,
-  SHA512C04=0x3956c25bf348b538,
-  SHA512C05=0x59f111f1b605d019,
-  SHA512C06=0x923f82a4af194f9b,
-  SHA512C07=0xab1c5ed5da6d8118,
-  SHA512C08=0xd807aa98a3030242,
-  SHA512C09=0x12835b0145706fbe,
-  SHA512C0a=0x243185be4ee4b28c,
-  SHA512C0b=0x550c7dc3d5ffb4e2,
-  SHA512C0c=0x72be5d74f27b896f,
-  SHA512C0d=0x80deb1fe3b1696b1,
-  SHA512C0e=0x9bdc06a725c71235,
-  SHA512C0f=0xc19bf174cf692694,
-  SHA512C10=0xe49b69c19ef14ad2,
-  SHA512C11=0xefbe4786384f25e3,
-  SHA512C12=0x0fc19dc68b8cd5b5,
-  SHA512C13=0x240ca1cc77ac9c65,
-  SHA512C14=0x2de92c6f592b0275,
-  SHA512C15=0x4a7484aa6ea6e483,
-  SHA512C16=0x5cb0a9dcbd41fbd4,
-  SHA512C17=0x76f988da831153b5,
-  SHA512C18=0x983e5152ee66dfab,
-  SHA512C19=0xa831c66d2db43210,
-  SHA512C1a=0xb00327c898fb213f,
-  SHA512C1b=0xbf597fc7beef0ee4,
-  SHA512C1c=0xc6e00bf33da88fc2,
-  SHA512C1d=0xd5a79147930aa725,
-  SHA512C1e=0x06ca6351e003826f,
-  SHA512C1f=0x142929670a0e6e70,
-  SHA512C20=0x27b70a8546d22ffc,
-  SHA512C21=0x2e1b21385c26c926,
-  SHA512C22=0x4d2c6dfc5ac42aed,
-  SHA512C23=0x53380d139d95b3df,
-  SHA512C24=0x650a73548baf63de,
-  SHA512C25=0x766a0abb3c77b2a8,
-  SHA512C26=0x81c2c92e47edaee6,
-  SHA512C27=0x92722c851482353b,
-  SHA512C28=0xa2bfe8a14cf10364,
-  SHA512C29=0xa81a664bbc423001,
-  SHA512C2a=0xc24b8b70d0f89791,
-  SHA512C2b=0xc76c51a30654be30,
-  SHA512C2c=0xd192e819d6ef5218,
-  SHA512C2d=0xd69906245565a910,
-  SHA512C2e=0xf40e35855771202a,
-  SHA512C2f=0x106aa07032bbd1b8,
-  SHA512C30=0x19a4c116b8d2d0c8,
-  SHA512C31=0x1e376c085141ab53,
-  SHA512C32=0x2748774cdf8eeb99,
-  SHA512C33=0x34b0bcb5e19b48a8,
-  SHA512C34=0x391c0cb3c5c95a63,
-  SHA512C35=0x4ed8aa4ae3418acb,
-  SHA512C36=0x5b9cca4f7763e373,
-  SHA512C37=0x682e6ff3d6b2b8a3,
-  SHA512C38=0x748f82ee5defb2fc,
-  SHA512C39=0x78a5636f43172f60,
-  SHA512C3a=0x84c87814a1f0ab72,
-  SHA512C3b=0x8cc702081a6439ec,
-  SHA512C3c=0x90befffa23631e28,
-  SHA512C3d=0xa4506cebde82bde9,
-  SHA512C3e=0xbef9a3f7b2c67915,
-  SHA512C3f=0xc67178f2e372532b,
-  SHA512C40=0xca273eceea26619c,
-  SHA512C41=0xd186b8c721c0c207,
-  SHA512C42=0xeada7dd6cde0eb1e,
-  SHA512C43=0xf57d4f7fee6ed178,
-  SHA512C44=0x06f067aa72176fba,
-  SHA512C45=0x0a637dc5a2c898a6,
-  SHA512C46=0x113f9804bef90dae,
-  SHA512C47=0x1b710b35131c471b,
-  SHA512C48=0x28db77f523047d84,
-  SHA512C49=0x32caab7b40c72493,
-  SHA512C4a=0x3c9ebe0a15c9bebc,
-  SHA512C4b=0x431d67c49c100d4c,
-  SHA512C4c=0x4cc5d4becb3e42b6,
-  SHA512C4d=0x597f299cfc657e2a,
-  SHA512C4e=0x5fcb6fab3ad6faec,
-  SHA512C4f=0x6c44198c4a475817
+  SHA512C00=0x428a2f98d728ae22UL,
+  SHA512C01=0x7137449123ef65cdUL,
+  SHA512C02=0xb5c0fbcfec4d3b2fUL,
+  SHA512C03=0xe9b5dba58189dbbcUL,
+  SHA512C04=0x3956c25bf348b538UL,
+  SHA512C05=0x59f111f1b605d019UL,
+  SHA512C06=0x923f82a4af194f9bUL,
+  SHA512C07=0xab1c5ed5da6d8118UL,
+  SHA512C08=0xd807aa98a3030242UL,
+  SHA512C09=0x12835b0145706fbeUL,
+  SHA512C0a=0x243185be4ee4b28cUL,
+  SHA512C0b=0x550c7dc3d5ffb4e2UL,
+  SHA512C0c=0x72be5d74f27b896fUL,
+  SHA512C0d=0x80deb1fe3b1696b1UL,
+  SHA512C0e=0x9bdc06a725c71235UL,
+  SHA512C0f=0xc19bf174cf692694UL,
+  SHA512C10=0xe49b69c19ef14ad2UL,
+  SHA512C11=0xefbe4786384f25e3UL,
+  SHA512C12=0x0fc19dc68b8cd5b5UL,
+  SHA512C13=0x240ca1cc77ac9c65UL,
+  SHA512C14=0x2de92c6f592b0275UL,
+  SHA512C15=0x4a7484aa6ea6e483UL,
+  SHA512C16=0x5cb0a9dcbd41fbd4UL,
+  SHA512C17=0x76f988da831153b5UL,
+  SHA512C18=0x983e5152ee66dfabUL,
+  SHA512C19=0xa831c66d2db43210UL,
+  SHA512C1a=0xb00327c898fb213fUL,
+  SHA512C1b=0xbf597fc7beef0ee4UL,
+  SHA512C1c=0xc6e00bf33da88fc2UL,
+  SHA512C1d=0xd5a79147930aa725UL,
+  SHA512C1e=0x06ca6351e003826fUL,
+  SHA512C1f=0x142929670a0e6e70UL,
+  SHA512C20=0x27b70a8546d22ffcUL,
+  SHA512C21=0x2e1b21385c26c926UL,
+  SHA512C22=0x4d2c6dfc5ac42aedUL,
+  SHA512C23=0x53380d139d95b3dfUL,
+  SHA512C24=0x650a73548baf63deUL,
+  SHA512C25=0x766a0abb3c77b2a8UL,
+  SHA512C26=0x81c2c92e47edaee6UL,
+  SHA512C27=0x92722c851482353bUL,
+  SHA512C28=0xa2bfe8a14cf10364UL,
+  SHA512C29=0xa81a664bbc423001UL,
+  SHA512C2a=0xc24b8b70d0f89791UL,
+  SHA512C2b=0xc76c51a30654be30UL,
+  SHA512C2c=0xd192e819d6ef5218UL,
+  SHA512C2d=0xd69906245565a910UL,
+  SHA512C2e=0xf40e35855771202aUL,
+  SHA512C2f=0x106aa07032bbd1b8UL,
+  SHA512C30=0x19a4c116b8d2d0c8UL,
+  SHA512C31=0x1e376c085141ab53UL,
+  SHA512C32=0x2748774cdf8eeb99UL,
+  SHA512C33=0x34b0bcb5e19b48a8UL,
+  SHA512C34=0x391c0cb3c5c95a63UL,
+  SHA512C35=0x4ed8aa4ae3418acbUL,
+  SHA512C36=0x5b9cca4f7763e373UL,
+  SHA512C37=0x682e6ff3d6b2b8a3UL,
+  SHA512C38=0x748f82ee5defb2fcUL,
+  SHA512C39=0x78a5636f43172f60UL,
+  SHA512C3a=0x84c87814a1f0ab72UL,
+  SHA512C3b=0x8cc702081a6439ecUL,
+  SHA512C3c=0x90befffa23631e28UL,
+  SHA512C3d=0xa4506cebde82bde9UL,
+  SHA512C3e=0xbef9a3f7b2c67915UL,
+  SHA512C3f=0xc67178f2e372532bUL,
+  SHA512C40=0xca273eceea26619cUL,
+  SHA512C41=0xd186b8c721c0c207UL,
+  SHA512C42=0xeada7dd6cde0eb1eUL,
+  SHA512C43=0xf57d4f7fee6ed178UL,
+  SHA512C44=0x06f067aa72176fbaUL,
+  SHA512C45=0x0a637dc5a2c898a6UL,
+  SHA512C46=0x113f9804bef90daeUL,
+  SHA512C47=0x1b710b35131c471bUL,
+  SHA512C48=0x28db77f523047d84UL,
+  SHA512C49=0x32caab7b40c72493UL,
+  SHA512C4a=0x3c9ebe0a15c9bebcUL,
+  SHA512C4b=0x431d67c49c100d4cUL,
+  SHA512C4c=0x4cc5d4becb3e42b6UL,
+  SHA512C4d=0x597f299cfc657e2aUL,
+  SHA512C4e=0x5fcb6fab3ad6faecUL,
+  SHA512C4f=0x6c44198c4a475817UL
 
 } sha2_64_constants_t;
 
 typedef enum ripemd160_constants
 {
-  RIPEMD160M_A=0x67452301,
-  RIPEMD160M_B=0xefcdab89,
-  RIPEMD160M_C=0x98badcfe,
-  RIPEMD160M_D=0x10325476,
-  RIPEMD160M_E=0xc3d2e1f0,
+  RIPEMD160M_A=0x67452301U,
+  RIPEMD160M_B=0xefcdab89U,
+  RIPEMD160M_C=0x98badcfeU,
+  RIPEMD160M_D=0x10325476U,
+  RIPEMD160M_E=0xc3d2e1f0U,
 
-  RIPEMD160C00=0x00000000,
-  RIPEMD160C10=0x5a827999,
-  RIPEMD160C20=0x6ed9eba1,
-  RIPEMD160C30=0x8f1bbcdc,
-  RIPEMD160C40=0xa953fd4e,
-  RIPEMD160C50=0x50a28be6,
-  RIPEMD160C60=0x5c4dd124,
-  RIPEMD160C70=0x6d703ef3,
-  RIPEMD160C80=0x7a6d76e9,
-  RIPEMD160C90=0x00000000,
+  RIPEMD160C00=0x00000000U,
+  RIPEMD160C10=0x5a827999U,
+  RIPEMD160C20=0x6ed9eba1U,
+  RIPEMD160C30=0x8f1bbcdcU,
+  RIPEMD160C40=0xa953fd4eU,
+  RIPEMD160C50=0x50a28be6U,
+  RIPEMD160C60=0x5c4dd124U,
+  RIPEMD160C70=0x6d703ef3U,
+  RIPEMD160C80=0x7a6d76e9U,
+  RIPEMD160C90=0x00000000U,
 
   RIPEMD160S00=11,
   RIPEMD160S01=14,
@@ -1469,36 +1512,36 @@ typedef enum ripemd160_constants
   RIPEMD160S9C=15,
   RIPEMD160S9D=13,
   RIPEMD160S9E=11,
-  RIPEMD160S9F=11u
+  RIPEMD160S9F=11
 
 } ripemd160_constants_t;
 
 typedef enum keccak_constants
 {
-  KECCAK_RNDC_00=0x0000000000000001,
-  KECCAK_RNDC_01=0x0000000000008082,
-  KECCAK_RNDC_02=0x000000000000808a,
-  KECCAK_RNDC_03=0x0000000080008000,
-  KECCAK_RNDC_04=0x000000000000808b,
-  KECCAK_RNDC_05=0x0000000080000001,
-  KECCAK_RNDC_06=0x0000000080008081,
-  KECCAK_RNDC_07=0x0000000000008009,
-  KECCAK_RNDC_08=0x000000000000008a,
-  KECCAK_RNDC_09=0x0000000000000088,
-  KECCAK_RNDC_10=0x0000000080008009,
-  KECCAK_RNDC_11=0x000000008000000a,
-  KECCAK_RNDC_12=0x000000008000808b,
-  KECCAK_RNDC_13=0x000000000000008b,
-  KECCAK_RNDC_14=0x0000000000008089,
-  KECCAK_RNDC_15=0x0000000000008003,
-  KECCAK_RNDC_16=0x0000000000008002,
-  KECCAK_RNDC_17=0x0000000000000080,
-  KECCAK_RNDC_18=0x000000000000800a,
-  KECCAK_RNDC_19=0x000000008000000a,
-  KECCAK_RNDC_20=0x0000000080008081,
-  KECCAK_RNDC_21=0x0000000000008080,
-  KECCAK_RNDC_22=0x0000000080000001,
-  KECCAK_RNDC_23=0x0000000080008008,
+  KECCAK_RNDC_00=0x0000000000000001UL,
+  KECCAK_RNDC_01=0x0000000000008082UL,
+  KECCAK_RNDC_02=0x800000000000808aUL,
+  KECCAK_RNDC_03=0x8000000080008000UL,
+  KECCAK_RNDC_04=0x000000000000808bUL,
+  KECCAK_RNDC_05=0x0000000080000001UL,
+  KECCAK_RNDC_06=0x8000000080008081UL,
+  KECCAK_RNDC_07=0x8000000000008009UL,
+  KECCAK_RNDC_08=0x000000000000008aUL,
+  KECCAK_RNDC_09=0x0000000000000088UL,
+  KECCAK_RNDC_10=0x0000000080008009UL,
+  KECCAK_RNDC_11=0x000000008000000aUL,
+  KECCAK_RNDC_12=0x000000008000808bUL,
+  KECCAK_RNDC_13=0x800000000000008bUL,
+  KECCAK_RNDC_14=0x8000000000008089UL,
+  KECCAK_RNDC_15=0x8000000000008003UL,
+  KECCAK_RNDC_16=0x8000000000008002UL,
+  KECCAK_RNDC_17=0x8000000000000080UL,
+  KECCAK_RNDC_18=0x000000000000800aUL,
+  KECCAK_RNDC_19=0x800000008000000aUL,
+  KECCAK_RNDC_20=0x8000000080008081UL,
+  KECCAK_RNDC_21=0x8000000000008080UL,
+  KECCAK_RNDC_22=0x0000000080000001UL,
+  KECCAK_RNDC_23=0x8000000080008008UL,
 
   KECCAK_PILN_00=10,
   KECCAK_PILN_01=7,
@@ -1554,32 +1597,32 @@ typedef enum keccak_constants
 
 typedef enum mysql323_constants
 {
-  MYSQL323_A=0x50305735,
-  MYSQL323_B=0x12345671u
+  MYSQL323_A=0x50305735U,
+  MYSQL323_B=0x12345671U
 
 } mysql323_constants_t;
 
 typedef enum fortigate_constants
 {
-  FORTIGATE_A=0x2eba88a3,
-  FORTIGATE_B=0x4ab04c42,
-  FORTIGATE_C=0xc1307953,
-  FORTIGATE_D=0x3fcc0731,
-  FORTIGATE_E=0x299032a1,
-  FORTIGATE_F=0x705b81a9
+  FORTIGATE_A=0x2eba88a3U,
+  FORTIGATE_B=0x4ab04c42U,
+  FORTIGATE_C=0xc1307953U,
+  FORTIGATE_D=0x3fcc0731U,
+  FORTIGATE_E=0x299032a1U,
+  FORTIGATE_F=0x705b81a9U
 
 } fortigate_constants_t;
 
 typedef enum blake2b_constants
 {
-  BLAKE2B_IV_00=0x6a09e667f3bcc908,
-  BLAKE2B_IV_01=0xbb67ae8584caa73b,
-  BLAKE2B_IV_02=0x3c6ef372fe94f82b,
-  BLAKE2B_IV_03=0xa54ff53a5f1d36f1,
-  BLAKE2B_IV_04=0x510e527fade682d1,
-  BLAKE2B_IV_05=0x9b05688c2b3e6c1f,
-  BLAKE2B_IV_06=0x1f83d9abfb41bd6b,
-  BLAKE2B_IV_07=0x5be0cd19137e2179
+  BLAKE2B_IV_00=0x6a09e667f3bcc908UL,
+  BLAKE2B_IV_01=0xbb67ae8584caa73bUL,
+  BLAKE2B_IV_02=0x3c6ef372fe94f82bUL,
+  BLAKE2B_IV_03=0xa54ff53a5f1d36f1UL,
+  BLAKE2B_IV_04=0x510e527fade682d1UL,
+  BLAKE2B_IV_05=0x9b05688c2b3e6c1fUL,
+  BLAKE2B_IV_06=0x1f83d9abfb41bd6bUL,
+  BLAKE2B_IV_07=0x5be0cd19137e2179UL
 
 } blake2b_constants_t;
 
@@ -1608,6 +1651,9 @@ typedef struct salt
   u32 salt_iter;
   u32 salt_iter2;
   u32 salt_sign[2];
+  u32 salt_repeats;
+
+  u32 orig_pos;
 
   u32 digests_cnt;
   u32 digests_done;
@@ -1689,4 +1735,13 @@ typedef struct keyboard_layout_mapping
 
 } keyboard_layout_mapping_t;
 
+typedef struct hc_enc
+{
+  int  pos;   // source offset
+
+  u32  cbuf;  // carry buffer
+  int  clen;  // carry length
+
+} hc_enc_t;
+
 #endif
